import 'dart:convert';
import 'dart:io';

import 'http_response.dart';

/// 使用方便的 HTTPClient 类
class HTTPClient {
  final HttpClient _client;

  /// 构造函数
  HTTPClient({SecurityContext context})
      : _client = HttpClient(context: context);

  /// 使用 GET 方式发起网络请求
  Future<HTTPResponse> getAsync(String uri) async {
    final url = Uri.parse(uri);
    final request = await _client.getUrl(url);
    final response = await request.close();
    final statusCode = response.statusCode;
    final content = await response.transform(utf8.decoder).join();
    final response1 = HTTPResponse(statusCode, content);
    return response1;
  }

  /// 使用 POST 方式发起网络请求，默认 ContentType 为 JSON 格式
  Future<HTTPResponse> postAsync(String uri, Object content,
      {ContentType contentType}) async {
    final url = Uri.parse(uri);
    final request = await _client.postUrl(url);
    request.headers.contentType = contentType ?? ContentType.json;
    request.write(content);
    final response = await request.close();
    final statusCode = response.statusCode;
    final content1 = await response.transform(utf8.decoder).join();
    final response1 = HTTPResponse(statusCode, content1);
    return response1;
  }

  /// 关闭 HTTPClient 并释放资源
  ///
  /// 如果 [force] = false，会等待所有网络请求完成后再关闭连接
  ///
  /// 如果 [force] = true，会立即释放连接，若有未完成的网络请求会抛出异常
  void close({bool force = false}) {
    _client.close(force: force);
  }
}
